package cn.iocoder.yudao.module.member.service.memberUserScoreDetail;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreDetailUpdateReqDto;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreDetail.MemberUserScoreDetailDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreDetail.MemberUserScoreDetailMapper;
import cn.iocoder.yudao.module.member.enums.MemberScoreStatueEnum;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.addDays;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.getNextNDayStart;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_SCORE_NOT_ENOUGH;

/**
 * 会员积分详情 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreDetailServiceImpl extends AbstractService<MemberUserScoreDetailMapper, MemberUserScoreDetailDO> implements MemberUserScoreDetailService {
    @Resource
    private MemberUserScoreDetailMapper detailMapper;

    @Override
    public void updateScoreDetail(MemberUserScoreDetailUpdateReqDto reqDto) {
        // 增加积分，只需要新增一条记录
        // 减少积分，使已有记录失效
        if (reqDto.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        if (reqDto.getOperateType() == ScoreOperateTypeEnum.ADD) {
            update4Add(reqDto);
        }
        if (reqDto.getOperateType() == ScoreOperateTypeEnum.REDUCE) {
            update4Reduce(reqDto);
        }
    }

    private void update4Reduce(MemberUserScoreDetailUpdateReqDto reqDto) {
        Long scoreCount = detailMapper.getScoreTotalByStatus(reqDto.getMemberId(), Lists.newArrayList(MemberScoreStatueEnum.AVAILABLE.getValue(), MemberScoreStatueEnum.PART_AVAILABLE.getValue()));
        if (scoreCount == null || scoreCount < reqDto.getScoreCount()) {
            throw exception(MEMBER_SCORE_NOT_ENOUGH);
        }
        LambdaQueryWrapper<MemberUserScoreDetailDO> wrappers = Wrappers.lambdaQuery();
        wrappers.eq(MemberUserScoreDetailDO::getMemberId, reqDto.getMemberId());
        wrappers.in(MemberUserScoreDetailDO::getStatus, Lists.newArrayList(MemberScoreStatueEnum.AVAILABLE.getValue(), MemberScoreStatueEnum.PART_AVAILABLE.getValue()));
        wrappers.orderByAsc(MemberUserScoreDetailDO::getExpireTime);
        List<MemberUserScoreDetailDO> detailList = this.list(wrappers);
        int needCount = reqDto.getScoreCount();
        List<MemberUserScoreDetailDO> updateDetailList = new ArrayList<>();
        for (MemberUserScoreDetailDO memberUserScoreDetailDO : detailList) {
            if (needCount == 0) {
                break;
            }
            if (memberUserScoreDetailDO.getRemainCount() <= needCount) {
                needCount -= memberUserScoreDetailDO.getRemainCount();
                memberUserScoreDetailDO.setRemainCount(0);
                memberUserScoreDetailDO.setStatus(MemberScoreStatueEnum.NOT_AVAILABLE.getValue());
                memberUserScoreDetailDO.setExtParam(addScoreLogId(memberUserScoreDetailDO.getExtParam(), reqDto.getScoreLogId()));
                updateDetailList.add(memberUserScoreDetailDO);
            } else {
                memberUserScoreDetailDO.setRemainCount(memberUserScoreDetailDO.getRemainCount() - needCount);
                needCount = 0;
                memberUserScoreDetailDO.setStatus(MemberScoreStatueEnum.PART_AVAILABLE.getValue());
                memberUserScoreDetailDO.setExtParam(addScoreLogId(memberUserScoreDetailDO.getExtParam(), reqDto.getScoreLogId()));
                updateDetailList.add(memberUserScoreDetailDO);
            }
        }
        this.updateBatch(updateDetailList);
    }

    private void update4Add(MemberUserScoreDetailUpdateReqDto reqDto) {
        MemberUserScoreDetailDO memberUserScoreDetailDO = new MemberUserScoreDetailDO();
        memberUserScoreDetailDO.setMemberId(reqDto.getMemberId());
        memberUserScoreDetailDO.setScoreCount(reqDto.getScoreCount());
        memberUserScoreDetailDO.setStatus(MemberScoreStatueEnum.AVAILABLE.getValue());
        memberUserScoreDetailDO.setRemainCount(reqDto.getScoreCount());
        memberUserScoreDetailDO.setCreateTime(new Date());
        if (reqDto.getExpireDays() != null) {
            memberUserScoreDetailDO.setExpireTime(addDays(getNextNDayStart(memberUserScoreDetailDO.getCreateTime(), 1), reqDto.getExpireDays()));
        }
        Map<String, Object> extParma = new HashMap<>();
        extParma.put(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), Lists.newArrayList(reqDto.getScoreLogId()));
        memberUserScoreDetailDO.setExtParam(JSONUtil.toJsonStr(extParma));
        this.saveOrUpdate(memberUserScoreDetailDO);
    }

    private String addScoreLogId(String extParam, Long scoreLogId) {
        JSONObject extParamJson = JSONUtil.parseObj(extParam);
        if (extParam == null ){
            extParamJson = new JSONObject();
        }
        List<Long> logIds = extParamJson.get(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), List.class);
        if (logIds == null) {
            logIds = new ArrayList<>();
        }
        logIds.add(scoreLogId);
        extParamJson.set(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), logIds);
        return extParamJson.toStringPretty();
    }
}
